﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace CSharpCode
{
    /// <summary>
    /// Summary description for ImmutableAccount
    /// </summary>
    [TestClass]
    public class ImmutableAccount
    {
        public class Account
        {
            public Account(decimal balance)
            {
                Balance = balance;
            }

            public decimal Balance { get; private set; }
        }

        private Account ReceivePayCheck(Account account)
        {
            return new Account(account.Balance + 1000);
        }

        private Account PayBills(Account account)
        {
            if (account.Balance < 700)
                throw new ArgumentException("Insufficent funds to pay bills", "account");

            return new Account(account.Balance - 700);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void BillsFirst()
        {
            //still raises an ArgumentException
            var account = new Account(100);

            PayBills(account);
            ReceivePayCheck(account);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void PayCheckFirst()
        {
            //now this raises an ArgumentException, too!
            var account = new Account(100);

            ReceivePayCheck(account);
            PayBills(account);
        }

        [TestMethod]
        public void ExplicitDependency()
        {
            var account = new Account(100);
            PayBills(ReceivePayCheck(account));
        }

    }
}
